/*******************************************************************************
* Final Project : ShadersProject
*
*
* Author: Joseph Soldo
* Class:  CSC 471-01
*******************************************************************************
*/
#define _USE_MATH_DEFINES
#define GLH_EXT_SINGLE_FILE

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <vector>
#include <iostream>
#include <fstream>
#include <GL/glut.h>
#include <GL/gl.h>
#include <assert.h>
#include <map>
//cg header files
#include <cg/cg.h>
#include <cg/cgGL.h>

#include "glh_extensions.h"

//devIL header files
#include <il/il.h>
#include <il/ilu.h>
#include <il/ilut.h>

//glui header
#include "glui.h"

using namespace std;

//GLOBAL VARIABLES

//globals for reshape
int GW;
int GH;

float R = 2.0;
float L = -2.0;
float T = 2.0;
float B = -2.0;

//id used by glui
int mainWindowID;

//global for controling transformation modes
enum transformMode {transMesh, rotateMesh, scaleMesh, transLight, depthLight, rotateCamera};
transformMode mode = rotateCamera;   //default to start in this mode

//global to control which aspect of lighting to change
enum lightMode {amb, spec, diff};
lightMode lighting = spec;


//data strucutre to store 3d points
struct Vector3{
  float x;
  float y;
  float z;
  
} ;

//globals for camera movements
Vector3 eye;
Vector3 lookat;
Vector3 up;
Vector3 gaze;

float CameraTheta = 90;
float CameraPhi = 0;
float CameraRadius = 3;

//a struct to hold data for materials
typedef struct materialStruct {
  GLfloat ambient[4];
  GLfloat diffuse[4];
  GLfloat specular[4];
  GLfloat shininess[1];
  bool set;
} materialStruct;

//data structure to store triangle - note that v1, v2, and v3 are indexes into the vertex array
typedef struct Tri{
  int v1;
  int v2;
  int v3;
  Vector3 normal;
  Vector3 color;
  Vector3 center;
  materialStruct material;
} Tri;

//data structure to store verticies and their normals
typedef struct Vertex{
	Vector3 point;
	Vector3 normal;
} Vertex;


//a struct to hold various information about how to draw an object so that
//different objects don't have the same xforms and styles applied to them
struct objectVars{
	bool mesh;
	bool drawNormals;
	bool fillShape;
	bool smoothShade;
	bool shader;
	bool loaded;

	//variables for transformations
	float scale;
	float oldScale;
	float transX;
	float transY;

	//used for trackball rotation transforms
	GLfloat objectM[4][4];
	GLfloat *trackballM;

	//all the triangles in the mesh
	vector<Tri> Triangles;
	//all the vertices in the mesh
	vector<Vertex> Vertices;

	//for computing the center point and extent of the model
	Vector3 center;
	float max_x, max_y, max_z, min_x, min_y, min_z;
	float max_extent;
};

objectVars objects[3];

objectVars *cur;

float len; //global used in many places to speed up normalization of vectors

//variables for virtual trackball
int track;
int startx, starty, endx, endy;
float angle = 0.0;
Vector3 axis;
Vector3 v;
Vector3 u;


//global to control perspective or orthographic viewing
int view_mode;


//forward declarations of functions
void readLine(char* str);
void readStream(istream& is);
void drawTri(Tri);
void drawObjects();
void display();
void TrackBall();

//global to control if lighting should be used
int light;

//globals used to change aspects of lighting
GLfloat defaultPos[4] = {1.0, 1.0, 1.5, 0.0};
GLfloat defaultAmb[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat defaultDiff[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat defaultSpec[4] = {0.8, 0.8, 0.8, 1.0};

GLfloat light_pos[4] = {1.0, 1.0, 1.5, 0.0};
GLfloat light_amb[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat light_diff[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat light_spec[4] = {0.8, 0.8, 0.8, 1.0};

GLfloat *posp = light_pos;
GLfloat *ambp = light_amb;
GLfloat *diffp = light_diff;
GLfloat *specp = light_spec;

//global to choose which material to use if none were defined in the mesh file
int mat = 0;

//materials to use
materialStruct defMat = {
  {0.3, 0.0, 0.0, 1.0}, //amb
  {0.9, 0.0, 0.0, 1.0}, //diff
  {0.2, 0.0, 0.4, 1.0}, //spec
  {65.0} //shininess
};
materialStruct curMat = {
  {0.3, 0.0, 0.0, 1.0}, //amb
  {0.9, 0.0, 0.0, 1.0}, //diff
  {0.2, 0.0, 0.4, 1.0}, //spec
  {65.0} //shininess
};

//globals for the shaders
static CGcontext cgContext;
static CGprofile cgVertexProfile;
static CGprogram cgVertexProgram;
static CGprofile cgFragmentProfile;
static CGprogram cgFragmentProgram;

static char buffer[1024];
static GLuint cubemapTextureID;

CGparameter cgModelViewProj;
CGparameter cgModelViewIT;
CGparameter cgModelView;
CGparameter cgModelViewEyeI;
CGparameter cgEyePosition;
CGparameter cgEnvironmentMap;

CGparameter cgEtaRatio;
CGparameter cgReflectivity;
CGparameter cgMatRatio;
CGparameter cgWeight;

CGparameter cgSpecColor;
CGparameter cgDiffColor;
CGparameter cgLightPosition;
CGparameter cgGlobalAmbient;
CGparameter cgKs;
CGparameter cgKd;
CGparameter cgKa;
CGparameter cgShininess;
CGparameter cgAw;
CGparameter cgDw;
CGparameter cgShadowReduction;

//variables to pass to the shaders
float reflectivity = 1;
float matRatio = 1;
float ambWeight = 1;
float diffWeight = 1;
float weight = .5;
float eta = .9;
int shadowReduction = 1;

float defaultReflectivity = 1;
float defaultMatRatio = 1;
float defaultAmbWeight = 1;
float defaultDiffWeight = 1;
float defaultWeight = .5;
float defaultEta = .9;
int defaultShadowReduction = 1;


//forward declaration of some functions
static void setShaderParameters(void);
static void drawSkyBox(void);
static void loadCubeMap(const char *string);

//glui globals
GLUI *glui;
GLUI_Panel *masterPanel;
GLUI_Rollout *roll1;
GLUI_Rollout *roll2;
GLUI_Rollout *roll3;
GLUI_Panel *lightPan;
GLUI_Panel *matPan;
GLUI_Panel *meshPan;
GLUI_Panel *texPan;

bool lightChanged = false;
bool envChanged = false;
int meshChoice;
int envChoice;
GLUI_Listbox *meshList;
GLUI_RadioGroup *env;


//callback for glui functionality
void gluiCB(int o){
	int i = 0;

	switch (o){
		case 0:
			lightChanged = true;
			break;
		case 1:
			for(i = 0; i < 3; i++){
				light_spec[i] = defaultSpec[i];
				light_amb[i] = defaultAmb[i];
				light_diff[i] = defaultDiff[i];
			}	
			diffp = light_diff;
			ambp = light_amb;
			specp = light_spec;

			glui->sync_live();
			lightChanged = true;
			break;
		case 2:
			for(i = 0; i < 3; i++){
				curMat.ambient[i] = defMat.ambient[i];
				curMat.diffuse[i] = defMat.diffuse[i];
				curMat.specular[i] = defMat.specular[i];
			}
			curMat.shininess[0] = defMat.shininess[0];

			glui->sync_live();
			break;
		case 3:
			meshChoice = meshList->get_int_val();

			if(objects[meshChoice].loaded){
				cur = &objects[meshChoice];
			} else {
				cur = &objects[meshChoice];
				printf("Object not initialized. Defaulting to shpere.\n");
			}
			break;
		case 4:
			envChoice = env->get_int_val();
			printf("%d\n", envChoice);
			if(envChoice == 0){
				envChanged = true;
			}else if(envChoice == 1){
				envChanged = true;
			}else if(envChoice == 2){
				envChanged = true;
			}
			break;
		case 5:
			reflectivity = defaultReflectivity;
			matRatio = defaultMatRatio;
			ambWeight = defaultAmbWeight;
			diffWeight = defaultDiffWeight;
			weight = defaultWeight;
			eta = defaultEta = .98;
			shadowReduction = defaultShadowReduction;
			glui->sync_live();
			break;
		case 6:
			cur->shader = !cur->shader;
			break;
	}
}


//Function to create the entire glui control window
void createGLUI(){
	glui = GLUI_Master.create_glui( "Controls");

	//Light and Material Rollout Panel
	roll1 = glui->add_rollout( "Light and Material", true );
		//Light Panel
		lightPan = glui->add_panel_to_panel( roll1, "Light" );
			//ambient light
			glui->add_statictext_to_panel(lightPan, "Ambient");

			GLUI_Spinner *alrs = glui->add_spinner_to_panel( lightPan, "Red:",
				GLUI_SPINNER_FLOAT, &light_amb[0], 0, gluiCB);
			alrs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			alrs->set_speed(.4);

			GLUI_Spinner *algs = glui->add_spinner_to_panel( lightPan, "Green:",
				GLUI_SPINNER_FLOAT, &light_amb[1], 0, gluiCB);
			algs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			algs->set_speed(.4);

			GLUI_Spinner *albs = glui->add_spinner_to_panel( lightPan, "Blue:",
				GLUI_SPINNER_FLOAT, &light_amb[2], 0, gluiCB);
			albs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			albs->set_speed(.4);
			
			//diffuse light
			glui->add_statictext_to_panel(lightPan, "Diffuse");

			GLUI_Spinner *dlrs = glui->add_spinner_to_panel( lightPan, "Red:",
				GLUI_SPINNER_FLOAT, &light_diff[0], 0, gluiCB);
			dlrs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			dlrs->set_speed(.4);

			GLUI_Spinner *dlgs = glui->add_spinner_to_panel( lightPan, "Green:",
				GLUI_SPINNER_FLOAT, &light_diff[1], 0, gluiCB);
			dlgs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			dlgs->set_speed(.4);

			GLUI_Spinner *dlbs = glui->add_spinner_to_panel( lightPan, "Blue:",
				GLUI_SPINNER_FLOAT, &light_diff[2], 0, gluiCB);
			dlbs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			dlbs->set_speed(.4);

			//specular light
			glui->add_statictext_to_panel(lightPan, "Specular");

			GLUI_Spinner *slrs = glui->add_spinner_to_panel( lightPan, "Red:",
				GLUI_SPINNER_FLOAT, &light_spec[0], 0, gluiCB);
			slrs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			slrs->set_speed(.4);

			GLUI_Spinner *slgs = glui->add_spinner_to_panel( lightPan, "Green:",
				GLUI_SPINNER_FLOAT, &light_spec[1], 0, gluiCB);
			slgs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			slgs->set_speed(.4);

			GLUI_Spinner *slbs = glui->add_spinner_to_panel( lightPan, "Blue:",
				GLUI_SPINNER_FLOAT, &light_spec[2], 0, gluiCB);
			slbs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			slbs->set_speed(.4);

			//default button
			glui->add_button_to_panel(lightPan, "Default Light", 1, gluiCB);

		glui->add_column_to_panel(roll1, false);

		//Material Panel
		matPan = glui->add_panel_to_panel( roll1, "Material");
			//Ka aspect
			glui->add_statictext_to_panel(matPan, "Ka");

			GLUI_Spinner *kars = glui->add_spinner_to_panel( matPan, "Red:",
				GLUI_SPINNER_FLOAT, &curMat.ambient[0]);
			kars->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			kars->set_speed(.4);

			GLUI_Spinner *kags = glui->add_spinner_to_panel( matPan, "Green:",
				GLUI_SPINNER_FLOAT, &curMat.ambient[1]);
			kags->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			kags->set_speed(.4);

			GLUI_Spinner *kabs = glui->add_spinner_to_panel( matPan, "Blue:",
				GLUI_SPINNER_FLOAT, &curMat.ambient[2]);
			kabs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			kabs->set_speed(.4);
			
			//Kd aspect
			glui->add_statictext_to_panel(matPan, "Kd");

			GLUI_Spinner *kdrs = glui->add_spinner_to_panel( matPan, "Red:",
				GLUI_SPINNER_FLOAT, &curMat.diffuse[0]);
			kdrs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			kdrs->set_speed(.4);

			GLUI_Spinner *kdgs = glui->add_spinner_to_panel( matPan, "Green:",
				GLUI_SPINNER_FLOAT, &curMat.diffuse[1]);
			kdgs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			kdgs->set_speed(.4);

			GLUI_Spinner *kdbs = glui->add_spinner_to_panel( matPan, "Blue:",
				GLUI_SPINNER_FLOAT, &curMat.diffuse[2]);
			kdbs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			kdbs->set_speed(.4);

			//Ks aspect
			glui->add_statictext_to_panel(matPan, "Ks");

			GLUI_Spinner *ksrs = glui->add_spinner_to_panel( matPan, "Red:",
				GLUI_SPINNER_FLOAT, &curMat.specular[0]);
			ksrs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			ksrs->set_speed(.4);

			GLUI_Spinner *ksgs = glui->add_spinner_to_panel( matPan, "Green:",
				GLUI_SPINNER_FLOAT, &curMat.specular[1]);
			ksgs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			ksgs->set_speed(.4);

			GLUI_Spinner *ksbs = glui->add_spinner_to_panel( matPan, "Blue:",
				GLUI_SPINNER_FLOAT, &curMat.specular[2]);
			ksbs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
			ksbs->set_speed(.4);

			//Shininess aspect
			glui->add_statictext_to_panel(matPan, " ");
			
			GLUI_Spinner *shs = glui->add_spinner_to_panel( matPan, "Shininess:",
				GLUI_SPINNER_FLOAT, &curMat.shininess[0]);
			shs->set_float_limits(0.0, 100.0, GLUI_LIMIT_CLAMP);
			shs->set_speed(2);
			
			//Default Button
			glui->add_button_to_panel(matPan, "Default Material", 2, gluiCB);

	//mesh and texture rollout panel
	roll2 = glui->add_rollout( "Mesh and Environment Texture", true );
		//mesh panel
		meshPan = glui->add_panel_to_panel( roll2, "Mesh" );		
		
			meshList = glui->add_listbox_to_panel(meshPan, "Mesh:", NULL, 3, gluiCB);
			meshList->add_item(2, "Cessna");
			meshList->add_item(1, "Bunny");
			meshList->add_item(0, "Sphere");			
			

		glui->add_column_to_panel(roll2, true);

		//environment texture panel
		texPan = glui->add_panel_to_panel( roll2, "Texture" );

		env = glui->add_radiogroup_to_panel(texPan, NULL, 4, gluiCB);
		glui->add_radiobutton_to_group(env, "Sky1");
		glui->add_radiobutton_to_group(env, "Sky2");
		glui->add_radiobutton_to_group(env, "Lobby");

	//shader rollout panel
	roll3 = glui->add_rollout( "Shader Options", true );
		GLUI_Spinner *mats = glui->add_spinner_to_panel( roll3, "Phong Shading Amount:",
			GLUI_SPINNER_FLOAT, &matRatio);
		mats->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
		mats->set_speed(.8);

		GLUI_Spinner *ambws = glui->add_spinner_to_panel( roll3, "Ambient Component:",
			GLUI_SPINNER_FLOAT, &ambWeight);
		ambws->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
		ambws->set_speed(.8);

		GLUI_Spinner *diffws = glui->add_spinner_to_panel( roll3, "Diffuse Component:",
			GLUI_SPINNER_FLOAT, &diffWeight);
		diffws->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
		diffws->set_speed(.8);

		glui->add_separator_to_panel(roll3);

		GLUI_Spinner *weights = glui->add_spinner_to_panel( roll3, "Other Shading Amount:",
			GLUI_SPINNER_FLOAT, &weight);
		weights->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
		weights->set_speed(.8);
		
		GLUI_Spinner *relfs = glui->add_spinner_to_panel( roll3, "Reflectivity/Transmitance:",
			GLUI_SPINNER_FLOAT, &reflectivity);
		relfs->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
		relfs->set_speed(.8);

		GLUI_Spinner *etas = glui->add_spinner_to_panel( roll3, "Eta:",
			GLUI_SPINNER_FLOAT, &eta);
		etas->set_float_limits(0.0, 1.0, GLUI_LIMIT_CLAMP);
		etas->set_speed(.2);

		glui->add_checkbox_to_panel(roll3, "Shadow Reduction:", &shadowReduction);

		glui->add_button_to_panel(roll3, "Default Shading", 5, gluiCB);
		glui->add_button_to_panel(roll3, "Shader Toggle", 6, gluiCB);
}



//Initializing the cg shaders
static void initCG(void) {
	// init the context
	cgContext = cgCreateContext();

	// init the profile
	cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(cgVertexProfile);

	cgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(cgFragmentProfile);
	
	// load the program programs	
	cgFragmentProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE, "fp_reflection.cg", cgFragmentProfile, "fp_reflection", 0);

	cgVertexProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE, "vp_reflection.cg", cgVertexProfile, "vp_reflection", 0);
	
    // retrieve the parameters
	cgEtaRatio = cgGetNamedParameter(cgVertexProgram, "etaRatio");
	cgEyePosition = cgGetNamedParameter(cgVertexProgram, "eyePosition");
	cgModelViewProj = cgGetNamedParameter(cgVertexProgram, "modelViewProj");
	cgModelViewIT = cgGetNamedParameter(cgVertexProgram, "modelViewIT");
	cgModelView = cgGetNamedParameter(cgVertexProgram, "modelView");
	cgModelViewEyeI = cgGetNamedParameter(cgVertexProgram, "modelViewEyeI");
	if(!cgModelViewProj || !cgEyePosition || !cgModelViewIT || !cgModelView ||
				!cgEtaRatio || !cgModelViewEyeI){
		printf("Unable to get program parameters from vertex shader, exiting...\n");
		for(int i = 0; i < 2000000; i++)
			printf("\b");
		exit(0);
	}

	cgSpecColor = cgGetNamedParameter(cgVertexProgram, "specColor");
	cgDiffColor = cgGetNamedParameter(cgVertexProgram, "diffColor");
	cgLightPosition = cgGetNamedParameter(cgVertexProgram, "lightPosition");
	cgKs = cgGetNamedParameter(cgVertexProgram, "Ks");
	cgKd = cgGetNamedParameter(cgVertexProgram, "Kd");
	cgKa = cgGetNamedParameter(cgVertexProgram, "Ka");
	cgShininess = cgGetNamedParameter(cgVertexProgram, "shininess");
	cgGlobalAmbient = cgGetNamedParameter(cgVertexProgram, "globalAmbient");
	cgDw = cgGetNamedParameter(cgVertexProgram, "dw");
	cgAw = cgGetNamedParameter(cgVertexProgram, "aw");
	if(!cgSpecColor || !cgDiffColor || !cgLightPosition || !cgKs || !cgKd || !cgKa || !cgShininess 
			|| !cgGlobalAmbient || !cgAw || !cgDw ){
		printf("Unable to get program parameters from vertex shader, exiting...\n");
		for(int i = 0; i < 2000000; i++)
			printf("\b");
		exit(0);
	}

	cgEnvironmentMap = cgGetNamedParameter(cgFragmentProgram, "environmentMap");
	cgReflectivity = cgGetNamedParameter(cgFragmentProgram, "reflectivity");
	cgMatRatio = cgGetNamedParameter(cgFragmentProgram, "matRatio");
	cgWeight = cgGetNamedParameter(cgFragmentProgram, "weight");
	cgShadowReduction = cgGetNamedParameter(cgFragmentProgram, "shadowReduction");
	//if(!cgEnvironmentMap){
	if(!cgEnvironmentMap || !cgReflectivity || !cgMatRatio || !cgWeight || !cgShadowReduction){
		printf("Unable to get program parameters from fragment shader, exiting...\n");
		for(int i = 0; i < 2000000; i++)
			printf("\b");
		exit(0);
	}

	// load the programs
	cgGLLoadProgram(cgVertexProgram);
	cgGLLoadProgram(cgFragmentProgram);
}


//sets up a specific material
void materials(materialStruct *materials) {
	glMaterialfv(GL_FRONT, GL_AMBIENT, materials->ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, materials->diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, materials->specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, materials->shininess);
}


//initialization calls for opengl for static light
//note that we still need to enable lighting in order for it to work
//see keyboard 'l' event
void init_lighting() {
  //turn on light0
  glEnable(GL_LIGHT0);
  //set the light's position
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
  //set up the diffuse, ambient and specular components for the light
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);
  //specify our lighting model as 1 normal per vertex
  glShadeModel(GL_SMOOTH);
  cur->smoothShade = true;
}

//method to change lighting to the values specfied in the parameters passed in
void changeLight(GLfloat pos[4], GLfloat diff[4], GLfloat amb[4],GLfloat spec[4]){
	//set the light's position
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
	//set up the diffuse, ambient and specular components for the light
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diff);
	glLightfv(GL_LIGHT0, GL_AMBIENT, amb);
	glLightfv(GL_LIGHT0, GL_SPECULAR, spec);
}


//open the file for reading
int ReadFile(char* filename) {
  printf("Reading coordinates from %s\n", filename);
  
  ifstream in_f(filename);
  if (!in_f)  {
    printf("Could not open file %s\n", filename);
    return -1;
  } else {
    readStream(in_f);
	free(in_f);
	return 0;
  }  
}


//process the input stream from the file
void readStream(istream& is)
{
  char str[256];
  for (;is;) {
    is >> ws;
    is.get(str,sizeof(str));
    if (!is) break;
    is.ignore(9999,'\n');
    readLine(str);
  }
}


//method to calculate the dot product of 2 vectors
float dotProduct(Vector3 v1, Vector3 v2){
	return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

//method to calculate the cross product of 2 vectors
void crossProduct(Vector3 *a, Vector3 *b, Vector3 *result){
	result->x = a->y*b->z - a->z*b->y;
	result->y = a->z*b->x - a->x*b->z;
	result->z = a->x*b->y - a->y*b->x;
}

//method to calculate the magnitude of a vector
float magnitude(Vector3 v){
	return (float)sqrt(dotProduct(v, v));
}



//globals to use when calculating normals (more effiecient than if they were
//declared inside calcNormal method)
Vector3 a;
Vector3 b;
Vertex *v1;
Vertex *v2;
Vertex *v3;

//method to calculate the normal of a triangle and add the normal to the
//normals of its verticies.  It is used in initialization of the data structures
void calcNormal(Tri *t){
	//finding the normal with the cross product
	v1 = &(cur->Vertices.at(t->v1 - 1));
	v2 = &(cur->Vertices.at(t->v2 - 1));
	v3 = &(cur->Vertices.at(t->v3 - 1));

	a.x = v1->point.x - v2->point.x;
	a.y = v1->point.y - v2->point.y;
	a.z = v1->point.z - v2->point.z;

	b.x = v3->point.x - v2->point.x;
	b.y = v3->point.y - v2->point.y;
	b.z = v3->point.z - v2->point.z;

	crossProduct(&b, &a, &t->normal);

	//normalizing the normal
	float len = magnitude(t->normal);

	t->normal.x = t->normal.x/(len);
	t->normal.y = t->normal.y/(len);
	t->normal.z = t->normal.z/(len);

	//storing the center point of the triangle
	t->center.x = (v1->point.x + v2->point.x + v3->point.x)/3;
	t->center.y = (v1->point.y + v2->point.y + v3->point.y)/3;
	t->center.z = (v1->point.z + v2->point.z + v3->point.z)/3;

	//adding the normal to the normals of the tri's vertices
	v1->normal.x = v1->normal.x + t->normal.x;
	v1->normal.y = v1->normal.y + t->normal.y;
	v1->normal.z = v1->normal.z + t->normal.z;
	v2->normal.x = v2->normal.x + t->normal.x;
	v2->normal.y = v2->normal.y + t->normal.y;
	v2->normal.z = v2->normal.z + t->normal.z;
	v2->normal.x = v2->normal.x + t->normal.x;
	v2->normal.y = v2->normal.y + t->normal.y;
	v2->normal.z = v2->normal.z + t->normal.z;
}

//process each line of input save vertices and faces appropriately
void readLine(char* str) {  
  int indx = 0, vi;
  float x, y, z;
  float r, g, b;
  int mat;
  
  if (str[0]=='#') return;
  if (str[0]=='V' && !strncmp(str,"Vertex ",7)) {
    static Vertex v;
    assert(sscanf(str,"Vertex %d %g %g %g",&vi,&x,&y,&z)==4);
	v.point.x = x;
    v.point.y = y;
    v.point.z = z;
	v.normal.x = 0;
	v.normal.y = 0;
	v.normal.z = 0;
    cur->Vertices.push_back(v);
    //house keeping to display in center of the scene
	cur->center.x += v.point.x;
	cur->center.y += v.point.y;
	cur->center.z += v.point.z;
    if (v.point.x > cur->max_x) cur->max_x = v.point.x; 
	if (v.point.x < cur->min_x) cur->min_x = v.point.x;
    if (v.point.y > cur->max_y) cur->max_y = v.point.y; 
	if (v.point.y < cur->min_y) cur->min_y = v.point.y;
    if (v.point.z > cur->max_z) cur->max_z = v.point.z; 
	if (v.point.z < cur->min_z) cur->min_z = v.point.z;
  } else if (str[0]=='F' && !strncmp(str,"Face ",5)) {
    Tri t;
    char* s=str+4;
    int fi=-1;
    for (int t_i = 0;;t_i++) {
      while (*s && isspace(*s)) s++;
      //if we reach the end of the line break out of the loop
      if (!*s) break;
      //save the position of the current character
      char* beg=s;
      //advance to next space
      while (*s && isdigit(*s)) s++;
      //covert the character to an integer
      int j=atoi(beg);
      //the first number we encounter will be the face index, don't store it
      if (fi<0) { fi=j; continue; }
      //otherwise process the digit we've grabbed in j as a vertex index
      //the first number will be the face id the following are vertex ids
      if (t_i == 1)
        t.v1 = j;
      else if (t_i == 2)
        t.v2 = j;
      else if (t_i == 3)
        t.v3 = j;
      //if there is more data to process break out
      if (*s =='{') break;
    }
    //initialization - replace with real calculation
    calcNormal(&t);
	t.material.set = false;
    //possibly process colors if the mesh has colors
    if (*s && *s =='{'){
      char *s1 = s+1;
      //if we're reading off a color
      if (!strncmp(s1,"rgb=",4)) {
        //grab the values of the string
        assert(sscanf(s1,"rgb=(%g %g %g",&r,&g,&b)==3);
		if(sscanf(s1," matid=%d",&mat)== 1){
			//do nothing with the material id
		}
		//using the colors in the mesh to create a material
        t.color.x = r; t.color.x = g; t.color.x = b;
		t.material.ambient[0] = r*.45;
		t.material.ambient[1] = g*.45;
		t.material.ambient[2] = b*.45;
		t.material.ambient[3] = 1;
		t.material.diffuse[0] = r*.75;
		t.material.diffuse[1] = g*.75;
		t.material.diffuse[2] = b*.75;
		t.material.diffuse[3] = 1;
		t.material.specular[0] = r*1.0;
		t.material.specular[1] = g*1.0;
		t.material.specular[2] = b*1.0;
		t.material.specular[3] = 1;
		t.material.shininess[0] = 100;
		t.material.set = true;
	  }
	}
    cur->Triangles.push_back(t);
  }
}


//drawing routine to draw triangles as wireframe, solid color, or shaded
void drawTria(Tri t) {
	if(cur->fillShape){
		//determining which material to use
		if(t.material.set){
			materials(&t.material);
			cgGLSetParameter3f(cgKs, t.material.specular[0], t.material.specular[1],
				t.material.specular[2]);
			cgGLSetParameter3f(cgKd, t.material.diffuse[0], t.material.diffuse[1],
				t.material.diffuse[2]);
			cgGLSetParameter3f(cgKa, t.material.ambient[0], t.material.ambient[1],
				t.material.ambient[2]);
			cgGLSetParameter1f(cgShininess, t.material.shininess[0]);

		} else {
			materials(&curMat);
		}
		
		//drawing in shaded mode
		glNormal3f(t.normal.x, t.normal.y, t.normal.z);
		glBegin(GL_POLYGON);			
			glColor3f(0.0, 0.5, 0.5);
			glNormal3f(cur->Vertices[t.v1-1].normal.x, 
				cur->Vertices[t.v1-1].normal.y, 
				cur->Vertices[t.v1-1].normal.z);
			glVertex3f(cur->Vertices[t.v1 - 1].point.x, 
				cur->Vertices[t.v1 - 1].point.y,
				cur->Vertices[t.v1 - 1].point.z);
			glNormal3f(cur->Vertices[t.v2-1].normal.x, 
				cur->Vertices[t.v2-1].normal.y, 
				cur->Vertices[t.v2-1].normal.z);
			glVertex3f(cur->Vertices[t.v2 - 1].point.x, 
				cur->Vertices[t.v2 - 1].point.y,
				cur->Vertices[t.v2 - 1].point.z);
			glNormal3f(cur->Vertices[t.v3-1].normal.x, 
				cur->Vertices[t.v3-1].normal.y, 
				cur->Vertices[t.v3-1].normal.z);
			glVertex3f(cur->Vertices[t.v3 - 1].point.x, 
				cur->Vertices[t.v3 - 1].point.y,
				cur->Vertices[t.v3 - 1].point.z);
		glEnd();
	} else {   //drawing with a solid color
		glLineWidth(1.0);
		glBegin(GL_LINE_LOOP);
		glColor3f(0.0, 0.0, 0.5);
		//note that the vertices are indexed starting at 0, but the triangles
		//index them starting from 1, so we must offset by -1!!!
		glVertex3f(cur->Vertices[t.v1 - 1].point.x, 
			cur->Vertices[t.v1 - 1].point.y,
			cur->Vertices[t.v1 - 1].point.z);
		glVertex3f(cur->Vertices[t.v2 - 1].point.x, 
			cur->Vertices[t.v2 - 1].point.y,
			cur->Vertices[t.v2 - 1].point.z);
		glVertex3f(cur->Vertices[t.v3 - 1].point.x, 
			cur->Vertices[t.v3 - 1].point.y,
			cur->Vertices[t.v3 - 1].point.z);
		glEnd();
	}
  
	//drawing the triangle's normals
	glLineWidth(1.0);
	if(cur->drawNormals){
		glBegin(GL_LINES);
			glColor3f(1.0, 0.0, 0.0);
			glVertex3f(t.center.x, t.center.y, t.center.z);
			glVertex3f(t.center.x + t.normal.x/100, t.center.y + t.normal.y/100, 
				t.center.z + t.normal.z/100);
		glEnd();
	}
}


//method to draw a sphere
void drawSphere() {
	glColor3f(1.0, 0.0, 0.0);
	if(cur->fillShape){
		glutSolidSphere(.5, 100, 100);
	} else {
		glutWireSphere(.5, 100, 100);
	}
}



//method to draw the object
void drawObjects() {  
	//transforms for the mesh
	glPushMatrix();
	  
	//apply translation
	glTranslatef(cur->transX, cur->transY, 0);

	//rotating the mesh with trackball
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glRotatef(angle, axis.x, axis.y, axis.z);
	glMultMatrixf(cur->trackballM);
	glGetFloatv(GL_MODELVIEW_MATRIX, cur->trackballM);
	glPopMatrix();
	glMultMatrixf(cur->trackballM);
	  
	//scale object
	glScalef(cur->scale, cur->scale, cur->scale);
	  
	//translate the object to the orgin
	glTranslatef(-(cur->center.x), -(cur->center.y), -(cur->center.z));

	if(cur->shader){
		// Setting the parameters for the shaders
		setShaderParameters();
	}

	//draw the wireframe mesh
	if(cur->mesh){
		for(unsigned int j = 0; j < cur->Triangles.size(); j++) {
			drawTria(cur->Triangles[j]);
			materials(&curMat);
		}
	} else {
		drawSphere();
	}

	glPopMatrix();
}



//the display callback method 
void display(){  
	float numV = cur->Vertices.size();
	  
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	  
	glMatrixMode(GL_MODELVIEW);
	    
	glPushMatrix();
		glLoadIdentity();
		//set up the camera
		gluLookAt(eye.x, eye.y, eye.z, lookat.x, lookat.y, lookat.z, up.x, up.y, up.z);
	  
		cgGLSetStateMatrixParameter(cgModelViewEyeI, 
				CG_GL_MODELVIEW_MATRIX,CG_GL_MATRIX_INVERSE);

		//stablize the light to not rotate with camera
		glLightfv(GL_LIGHT0, GL_POSITION, light_pos);

		//draw the environment
		drawSkyBox();

		
		// Enable the profiles and binding the shader programs
		if(cur->shader){
			//cgGLLoadProgram(cgVertexProgram);
			//cgGLLoadProgram(cgFragmentProgram);
			cgGLEnableProfile(cgVertexProfile);
			cgGLEnableProfile(cgFragmentProfile);
			cgGLBindProgram(cgVertexProgram);
			cgGLBindProgram(cgFragmentProgram);
		}
	  
		glEnable(GL_CULL_FACE);
		drawObjects();
		glDisable(GL_CULL_FACE);
	glPopMatrix();
    
	glutSwapBuffers();

	if(cur->shader){
	  // Disabling the shader profiles
		cgGLDisableProfile(cgVertexProfile);
		cgGLDisableProfile(cgFragmentProfile);
		//cgGLUnloadProgram(cgVertexProgram);
	}
}


//reshape method called when the window is reshaped or the viewing mode is
//changed
void reshape(int w, int h) {
	GW = w;
	GH = h;
	  
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (view_mode == 0){
		glOrtho( L*(float)w/h, R*(float)w/h, B, T, 0.5, 40.0);
	} else {
		gluPerspective(45, (float)w/h, 0.5, 40.0);
	}
	  
	glMatrixMode(GL_MODELVIEW);
	glViewport(0, 0, w, h);

	if(cur->scale*cur->center.x+cur->transX > R*(float)GW/GH){
		cur->transX = R*(float)GW/GH - cur->scale*cur->center.x;
	} else if(cur->scale*cur->center.x + cur->transX < L*(float)GW/GH){
		cur->transX = L*(float)GW/GH - cur->scale * cur->center.x;
	}
	
	glutPostRedisplay();
}



//method to translate from pixel to world coordinate for the x value
float p2w_x(int p_x) {
	float xi = ( (float)p_x - ((GW-1.0)/2.0) )*2.0/GW;	
	return ((R-L)*xi + R + L)/2.0;
}


//method to translate from pixel to world coordinate for the y value
float p2w_y(int p_y) {
	return ((T-B)*((2.0 * (float)p_y)/GH) + T + B)/2;
}


//A simple function to translate x axis pixel coords to image coords.
float p2i_x(int px){
	return ((float)px - ((GW-1.0)/2.0))*2.0/GW;		
}


//A simple function to translate y axis pixel coords to image coords (note: it
//accounts for y value returned and inverts it automatically).
float p2i_y(int py){
	return (2.0 * (float)py - GH-1.0)/GH;	
}


//Method to calculate the angle between 2 vectors.  Result is returned in radians.
float calcAngle(Vector3 v1, Vector3 v2){
	float angle = dotProduct(v1, v2);
	angle = angle/(magnitude(v1) * magnitude(v2));
	return acos(angle) * 180/M_PI;
}


//Method to calculate the axis of rotation based on the first 2 vectors passed 
//in.  It stored the axis in the third vector passed in.
void calcVect(int x, int y, Vector3 *w){
	float xi = p2i_x(x);
	float yi = p2i_y(y);

	float l = sqrt(xi*xi + yi*yi);

	if(l > 2){
		w->x = xi/l;
		w->y = yi/l;
		w->z = 0;
	} else {
		w->x = xi;
		w->y = yi;
		w->z = sqrt(4-xi*xi-yi*yi);
	}
}


int mouseButton;

//mouse event function
void mouse(int button, int state, int x, int y) {
	mouseButton = button;
	if (button == GLUT_LEFT_BUTTON || button == GLUT_MIDDLE_BUTTON) {
		if (state == GLUT_DOWN) { /* if the left or middle button is clicked */
			startx = x;
			starty = GH-y-1;
		} else if (state == GLUT_UP) {
			angle = 0.0;	 
		}
	}
}


//mouse move function (where the bulk of transforms are performed)
void mouseMove(int x, int y) {
	if(mouseButton == GLUT_LEFT_BUTTON){
		//scaling the object
		if(mode == scaleMesh){
			cur->oldScale = cur->scale;
			cur->scale = cur->scale + 2.0*(p2i_x(x) - p2i_x(startx))/cur->max_extent;
			if(cur->scale < 0.00001){
				cur->scale = 0.00001;
			} else if (cur->scale > 4.0/(float)cur->max_extent){
				cur->scale = 4.0/(float)cur->max_extent;
			}
			startx = x;

			//renormalizing normals for a mesh
			if(cur->mesh){
				int size = (int)cur->Triangles.size();
				int i;
				float newScale = cur->scale/cur->oldScale;				
				
				for(i = 0; i < size; i++){
					cur->Triangles[i].normal.x = cur->Triangles[i].normal.x*newScale;
					cur->Triangles[i].normal.y = cur->Triangles[i].normal.y*newScale;
					cur->Triangles[i].normal.z = cur->Triangles[i].normal.z*newScale;
				}

				size = (int)cur->Vertices.size();

				for(i = 0; i < size; i++){
					cur->Vertices[i].normal.x = cur->Vertices[i].normal.x*newScale;
					cur->Vertices[i].normal.y = cur->Vertices[i].normal.y*newScale;
					cur->Vertices[i].normal.z = cur->Vertices[i].normal.z*newScale;
				}
			}

		//rotating the object using trackball
		} else if(mode == rotateMesh){
			endx = x;
			endy = GH-y-1;
			track = 1;
			calcVect(startx, starty, &u);
			calcVect(endx, endy, &v);
			crossProduct(&u, &v, &axis);
			angle = calcAngle(u, v);
			startx = x;
			starty = GH-y-1;

		//translating the object (x and y only)
		} else if(mode == transMesh){
			float dx, dy;
			dx = (p2w_x(x) - p2w_x(startx));
			dy = (p2w_y(GH-y-1) - p2w_y(starty));
			startx = x;
			starty = GH-y-1;
			cur->transX = cur->transX + dx;
			cur->transY = cur->transY + dy;

			if(cur->scale*cur->center.x+cur->transX > R*(float)GW/GH){
				cur->transX = R*(float)GW/GH - cur->scale*cur->center.x;
			} else if(cur->scale*cur->center.x + cur->transX < L*(float)GW/GH){
				cur->transX = L*(float)GW/GH - cur->scale * cur->center.x;
			}
			if(cur->scale*cur->center.y + cur->transY < B){
				cur->transY = B - cur->scale * cur->center.y;
			} else if(cur->scale*cur->center.y+cur->transY > T){
				cur->transY = T - cur->scale * cur->center.y;
			}
		
		//translating the light (x and y)
		} else if (mode == transLight){
			float dx, dy;
			dx = 2*(p2w_x(x) - p2w_x(startx));
			dy = 2*(p2w_y(GH-y-1) - p2w_y(starty));
			startx = x;
			starty = GH-y-1;
			posp[0] = posp[0] + dx;
			posp[1] = posp[1] + dy;

			if(posp[0] > 6){
				posp[0] = 6;
			} else if(posp[0] < -6){
				posp[0] = -6;
			}

			if(posp[1] > 6){
				posp[1] = 6;
			} else if(posp[1] < -6){
				posp[1] = -6;
			}

			changeLight(posp, diffp, ambp, specp);

		//translating the light in the z dirrection
		} else if (mode == depthLight){
			float dz;
			dz = 2*(p2w_y(starty)-p2w_y(GH-y-1));
			startx = x;
			starty = GH-y-1;
			posp[2] = posp[2] + dz;

			if(posp[2] > 6){
				posp[2] = 6;
			} else if(posp[2] < -6){
				posp[2] = -6;
			}

			changeLight(posp, diffp, ambp, specp);

		//rotating the camera around the origin
		} else if(mode == rotateCamera){
			float dx, dy;

			dx = (p2w_x(x) - p2w_x(startx));
			dy = (p2w_y(starty) - p2w_y(GH-y-1));
			startx = x;
			starty = GH-y-1;

			CameraTheta += dx * 45;
			CameraPhi += dy * 90;

			eye.x = CameraRadius * cos(CameraPhi *M_PI/180)* cos(CameraTheta*M_PI/180);
			eye.y = CameraRadius * sin(CameraPhi*M_PI/180);
			eye.z = CameraRadius * cos(CameraPhi*M_PI/180) * cos((90.0 - CameraTheta)*M_PI/180);

			gaze.x = lookat.x - eye.x;
			gaze.y = lookat.y - eye.y;
			gaze.z = lookat.z - eye.z;
			len = magnitude(gaze);
			gaze.x = gaze.x/len;
			gaze.y = gaze.y/len;
			gaze.z = gaze.z/len;		
		}		

		//zooming the camera
	} else if( mouseButton == GLUT_MIDDLE_BUTTON){
		float dx, dy, radius;
		Vector3 newEye;

		gaze.x = lookat.x - eye.x;
		gaze.y = lookat.y - eye.y;
		gaze.z = lookat.z - eye.z;
		len = magnitude(gaze);
		
		dx = 2*(p2w_x(x) - p2w_x(startx));
		dy = 2*(p2w_y(GH-y-1) - p2w_y(starty));
		startx = x;
		starty = GH-y-1;
		
		newEye.x = eye.x + (dx + dy)*gaze.x/len;
		newEye.y = eye.y + (dx + dy)*gaze.y/len;
		newEye.z = eye.z + (dx + dy)*gaze.z/len;

		radius = magnitude(newEye);

		if(radius > 1.5 && radius < 12.5){
			eye.x = eye.x + (dx + dy)*gaze.x/len;
			eye.y = eye.y + (dx + dy)*gaze.y/len;
			eye.z = eye.z + (dx + dy)*gaze.z/len;
			
			gaze.x = lookat.x - eye.x;
			gaze.y = lookat.y - eye.y;
			gaze.z = lookat.z - eye.z;
			len = magnitude(gaze);
			gaze.x = gaze.x/len;
			gaze.y = gaze.y/len;
			gaze.z = gaze.z/len;
			CameraRadius = len;				
		}
	}
	glutPostRedisplay();
}


//reseting all global variables that transform the mesh or light or change the
//aspects of the light
void reset(){
	int i;
	printf("Resetting all...\n");

	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX, cur->trackballM);

	cur->oldScale = cur->scale;
	if(cur->mesh){
		cur->scale = 1.5/(float)cur->max_extent;

		float newScale = cur->scale/cur->oldScale;

		int size = (int)cur->Triangles.size();

		for(i = 0; i < size; i++){
			cur->Triangles[i].normal.x = cur->Triangles[i].normal.x*newScale;
			cur->Triangles[i].normal.y = cur->Triangles[i].normal.y*newScale;
			cur->Triangles[i].normal.z = cur->Triangles[i].normal.z*newScale;
		}

		size = (int)cur->Vertices.size();

		for(i = 0; i < size; i++){
			cur->Vertices[i].normal.x = cur->Vertices[i].normal.x*newScale;
			cur->Vertices[i].normal.y = cur->Vertices[i].normal.y*newScale;
			cur->Vertices[i].normal.z = cur->Vertices[i].normal.z*newScale;
		}
	} else {
		cur->scale = 1;
	}

	cur->transX = 0;
	cur->transY = 0;

	for(i = 0; i < 3; i++){
		light_pos[i] = defaultPos[i];
		light_spec[i] = defaultSpec[i];
		light_amb[i] = defaultAmb[i];
		light_diff[i] = defaultDiff[i];
	}	

	posp = light_pos;
	diffp = light_diff;
	ambp = light_amb;
	specp = light_spec;

	changeLight(light_pos, light_diff, light_amb, light_spec);
}

//method to reset only global variables that affect the lighting
void resetLight(){
	printf("Resetting the Light...\n");

	int i = 0;

	for(i = 0; i < 3; i++){
		light_pos[i] = defaultPos[i];
		light_spec[i] = defaultSpec[i];
		light_amb[i] = defaultAmb[i];
		light_diff[i] = defaultDiff[i];
	}	

	posp = light_pos;
	diffp = light_diff;
	ambp = light_amb;
	specp = light_spec;

	changeLight(light_pos, light_diff, light_amb, light_spec);
}


//A simple function to print out the help screen.
void printHelp(){
	printf("\n---Help Menu---\n\nh - help menu\n");
	printf("q - quit\n\nAll other control aspects of the program are ");
	printf("either attached to the mouse\nbuttons or the control ");
	printf("window.  The menu attached to the right mouse button\nhas ");
	printf("the functions for the left mouse button, viewing options, and ");
	printf("other\nutility type functions.  The middle mouse button ");
	printf("controls camera zooming.");
	printf("\n\n---------------\n\n");
}


//method to translate various keys pressed into commands
void keyboard(unsigned char key, int x, int y) {
	switch(key) {
	case 'h': case 'H':
		printHelp();
		break;
	case 'q': case 'Q': case 27:
		exit(0);
		break;
	default:
		break;
	}
	glutPostRedisplay();
}



//The main menu.  Used by main to create the main menu that displays on a right
//click.  It contains the transformations, the reset commands, the help command
//and the exit command.
void mainMenu(int val){
	switch (val){
		case 1:
			mode = transMesh;
			printf("Translate Mode\n");			
			break;
		case 2:
			mode = scaleMesh;
			printf("Scale Mode\n");
			break;
		case 3:
			mode = rotateMesh;
			printf("Rotate Mode\n");
			break;
		case 4:
			mode = transLight;
			printf("Translate Light Mode\n");
			break;
		case 5:
			mode = depthLight;
			printf("Translate Light Depth Mode\n");
			break;
		case 7:
			mode = rotateCamera;
			printf("Camera Rotate Mode\n");
			break;
		case 8:
			printf("Resetting All...\n");
			reset();
			break;
		case 9:
			printf("Resetting lighting...\n");
			resetLight();
			break;
		case 10:
			printHelp();
			break;
		case 11:
			printf("Exiting...\n");
			exit(0);
			break;
	}
	glutPostRedisplay();
}



//The Viewing submenu.
void subMenu1(int val){
	switch (val){
		case 1:
			view_mode = !view_mode;
			reshape(GW, GH);
			break;
		case 2:
			cur->drawNormals = !cur->drawNormals;
			break;
		case 3:
			cur->fillShape = !cur->fillShape;
			break;
		case 4:
			cur->smoothShade = !cur->smoothShade;
			if(cur->smoothShade){
				glShadeModel(GL_SMOOTH);
			} else {
				glShadeModel(GL_FLAT);
			}
	}
	glutPostRedisplay();
}

// The error callback function for cg
static void cgErrorCallback(void) {		
	CGerror lastError = cgGetError();
	if(lastError){
		printf("%s\n", cgGetErrorString(lastError));
		printf("%s\n", cgGetLastListing(cgContext));
		printf("Cg error, exiting...\n");

		for(int i = 0; i < 2000000; i++)
			printf("\b");
		
		exit(0);
	}
}

//idle function needed for glui to work properly
void idle( void )
{
	/* According to the GLUT specification, the current window is 
		undefined during an idle callback.  So we need to explicitly change
		it if necessary */
	if ( glutGetWindow() != mainWindowID ) 
		glutSetWindow(mainWindowID);

	if(lightChanged){
		changeLight(light_pos, light_diff, light_amb, light_spec);
		lightChanged = false;
	}

	if(envChanged){
		if(envChoice == 0){
			loadCubeMap("sky");
		}else if(envChoice == 1){
			loadCubeMap("sky2");
		}else if(envChoice == 2){
			loadCubeMap("nvlobby");
		}
		envChanged = false;
	}
	glui->sync_live();
	glutPostRedisplay();
}



//the main method that initializes global variables as neccessary and starts
//the opengl loop
int main( int argc, char** argv ) {
	int menu, submenu1, returnVal;
	
	//set up my window
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(600, 600); 
	glutInitWindowPosition(100, 100);
	mainWindowID = glutCreateWindow("Mesh display");
	glClearColor(1.0, 1.0, 1.0, 1.0);
	  
	//register glut callback functions
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(mouseMove);
	//register cg error callback
	cgSetErrorCallback(cgErrorCallback);
	//register glui idle callback
	GLUI_Master.set_glutIdleFunc(idle);

	//setting up glui
	createGLUI();

	//printf("entered main\n");
	//check for multitexturing supported by the graphics card - if none exit
	//printf("%s\n", glGetString(GL_EXTENSIONS));

	//printf("got string \n");

	if(!glh_init_extensions("GL_ARB_multitexture")) {
		printf("Necessary extensions unsupported: %s\n",
			glh_get_unsupported_extensions());
		exit(0);
	}

	// initialize devIL
	ilInit(); 
	loadCubeMap("sky");
	    
	//set-up viewing
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-2.0, 2.0, -2.0, 2.0, 1.0, 15.0);
	glMatrixMode(GL_MODELVIEW);
	glViewport(0, 0, 300, 300); //image to pixel
  
	//initializations of globals	

	//initialization for sphere
	objects[0].drawNormals = false;
	objects[0].fillShape = true;
	objects[0].smoothShade = false;
	objects[0].mesh = false;
	objects[0].shader = false;
	objects[0].oldScale = 1;
	objects[0].scale = 1;	
	objects[0].transX = 0;
	objects[0].transY = 0;
	objects[0].max_x = 1;
	objects[0].max_y = 1;
	objects[0].max_z = 1;
	objects[0].min_x = -1;
	objects[0].min_y = -1;
	objects[0].min_z = -1;
	objects[0].max_extent = 2;
	objects[0].trackballM = (GLfloat *)objects[0].objectM;
	glMatrixMode(GL_MODELVIEW_MATRIX);
	glPushMatrix();
		glLoadIdentity();
		glGetFloatv(GL_MODELVIEW_MATRIX,objects[0].trackballM);
	glPopMatrix();
	objects[0].loaded = true;

	view_mode = 1;
	light = 1;	

	up.x = 0;
	up.y = 1;
	up.z = 0;
	eye.x = 0;
	eye.y = 0;
	eye.z = CameraRadius;
	lookat.x = 0;
	lookat.y = 0;
	lookat.z = 0;
	gaze.x = lookat.x - eye.x;
	gaze.y = lookat.y - eye.y;
	gaze.z = lookat.z - eye.z;
	len = magnitude(gaze);
	gaze.x = gaze.x/len;
	gaze.y = gaze.y/len;
	gaze.z = gaze.z/len;

	//create the menu
	submenu1 = glutCreateMenu(subMenu1);
	glutAddMenuEntry("Toggle Projection", 1);
	glutAddMenuEntry("Toggle Normals", 2);
	glutAddMenuEntry("Toggle Shape Filling", 3);
	glutAddMenuEntry("Toggle Smooth/Flat Shading", 4);

	menu = glutCreateMenu(mainMenu);
	glutAddSubMenu("Viewing", submenu1);
	glutAddMenuEntry("Translate Mesh", 1);
	glutAddMenuEntry("ScaleMesh", 2);
	glutAddMenuEntry("RotateMesh", 3);
	glutAddMenuEntry("Translate Light (xy)", 4);
	glutAddMenuEntry("Translate Light (z)", 5);
	glutAddMenuEntry("Rotate Camera", 7);
	glutAddMenuEntry("Reset All", 8);
	glutAddMenuEntry("Reset Light", 9);
	glutAddMenuEntry("Help", 10);
	glutAddMenuEntry("Exit", 11);

	glutAttachMenu(GLUT_RIGHT_BUTTON);  

	//initialization of meshes
	for(int i = 1; i < 3; i++){
		cur = &objects[i];
		cur->max_x = cur->max_y = cur->max_z = -999999;
		cur->min_x = cur->min_y = cur->min_z = 99999;
		cur->center.x = 0;
		cur->center.y = 0;
		cur->center.z = 0;
		cur->max_extent = 1.0;

		switch (i) {
			case 1:
				returnVal = ReadFile("bunny500.m");
				break;
			case 2:
				returnVal = ReadFile("cessna_color500.m");
				break;
		}
		if(returnVal == 0){
			meshChoice = i;
			cur->loaded = true;
			cur->transX = 0;
			cur->transY = 0;
			cur->mesh = true;
			cur->drawNormals = false;
			cur->fillShape = true;
			cur->smoothShade = false;
			cur->shader = true;
			cur->transX = 0;
			cur->transY = 0;
			cur->trackballM = (GLfloat *)&cur->objectM;
			glMatrixMode(GL_MODELVIEW_MATRIX);
			glPushMatrix();
				glLoadIdentity();
				glGetFloatv(GL_MODELVIEW_MATRIX,cur->trackballM);
			glPopMatrix();
			
			//once the file is parsed find out the maximum extent to center and 
			//scale mesh
			cur->max_extent = cur->max_x - cur->min_x;
			if (cur->max_y - cur->min_y > cur->max_extent){
				cur->max_extent = cur->max_y - cur->min_y;
			}
			cur->center.x = cur->center.x/cur->Vertices.size();
			cur->center.y = cur->center.y/cur->Vertices.size();
			cur->center.z = cur->center.z/cur->Vertices.size();

			//setting up the scale to fit the mesh into the window initially
			cur->scale = 1.5/(float)cur->max_extent;

			int size = (int)cur->Vertices.size();
			int i;
			float len;

			//normalizing the normals for the vertices
			for(i = 0; i < size; i++){
				len = magnitude(cur->Vertices[i].normal);
				cur->Vertices[i].normal.x = cur->Vertices[i].normal.x*cur->scale/len;
				cur->Vertices[i].normal.y = cur->Vertices[i].normal.y*cur->scale/len;
				cur->Vertices[i].normal.z = cur->Vertices[i].normal.z*cur->scale/len;
			}
		} else {
			cur->loaded = false;
		}
	}	

	//enable z-buffer
	glEnable(GL_DEPTH_TEST);

	//setting up lighting
	init_lighting();
	glEnable(GL_LIGHTING);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	initCG();

	//glEnable(GL_NORMALIZE);

	glutMainLoop();
}


// function used to read in an envirment cube map from 6 bmp files
static void loadCubeMap(const char *string) {
	// create cubeMap Idx and bind cubemap texture

	glGenTextures(1, &cubemapTextureID);
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubemapTextureID);

	
	// load cube map texture (6x)
	sprintf(buffer, "cubemaps/%s_posx.bmp", string);

	ilLoadImage(buffer);
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, 
				GL_RGB8,(ILfloat)ilGetInteger(IL_IMAGE_WIDTH), 
				(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB, 
				GL_UNSIGNED_BYTE, ilGetData() );


	sprintf(buffer, "cubemaps/%s_negx.bmp", string);
	ilLoadImage(buffer);
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB,
				GL_RGB8, (ILfloat)ilGetInteger(IL_IMAGE_WIDTH),
				(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB,
				GL_UNSIGNED_BYTE, ilGetData() );


	sprintf(buffer, "cubemaps/%s_posy.bmp", string);
	ilLoadImage(buffer);
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB,
				GL_RGB8, (ILfloat)ilGetInteger(IL_IMAGE_WIDTH),
				(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB,
				GL_UNSIGNED_BYTE, ilGetData() );


	sprintf(buffer, "cubemaps/%s_negy.bmp", string);
	ilLoadImage(buffer);
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB,
				GL_RGB8, (ILfloat)ilGetInteger(IL_IMAGE_WIDTH),
				(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB,
				GL_UNSIGNED_BYTE, ilGetData() );


	sprintf(buffer, "cubemaps/%s_posz.bmp", string);
	ilLoadImage(buffer);
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB,
				GL_RGB8, (ILfloat)ilGetInteger(IL_IMAGE_WIDTH),
				(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB,
				GL_UNSIGNED_BYTE, ilGetData() );


	sprintf(buffer, "cubemaps/%s_negz.bmp", string);
	ilLoadImage(buffer);
	ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
	gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB,
				GL_RGB8, (ILfloat)ilGetInteger(IL_IMAGE_WIDTH),
				(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB,
				GL_UNSIGNED_BYTE, ilGetData() );
		
}


// used to pass values into the shader
static void setShaderParameters( void ) {
	cgGLSetParameter4f(cgEyePosition, eye.x, eye.y, eye.z, 1);

	cgGLSetParameter1f(cgEtaRatio, eta);
	cgGLSetParameter1f(cgReflectivity, reflectivity);
	cgGLSetParameter1f(cgMatRatio, matRatio);

	cgGLSetParameter1f(cgShadowReduction, shadowReduction);

	cgGLSetParameter1f(cgDw, diffWeight);
	cgGLSetParameter1f(cgAw, ambWeight);
	cgGLSetParameter1f(cgWeight, weight);

	cgGLSetParameter4f(cgLightPosition, light_pos[0], light_pos[1], light_pos[2], 1);
	cgGLSetParameter3f(cgGlobalAmbient, light_amb[0], light_amb[1], light_amb[2]);
	cgGLSetParameter3f(cgSpecColor, light_spec[0], light_spec[1], light_spec[2]);
	cgGLSetParameter3f(cgDiffColor, light_diff[0], light_diff[1], light_diff[2]);

	cgGLSetParameter3f(cgKs, curMat.specular[0], curMat.specular[1], curMat.specular[2]);
	cgGLSetParameter3f(cgKd, curMat.diffuse[0], curMat.diffuse[1], curMat.diffuse[2]);
	cgGLSetParameter3f(cgKa, curMat.ambient[0], curMat.ambient[1], curMat.ambient[2]);
	cgGLSetParameter1f(cgShininess, curMat.shininess[0]);

	cgGLSetStateMatrixParameter(cgModelViewProj, 
				CG_GL_MODELVIEW_PROJECTION_MATRIX,CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelView,
				CG_GL_MODELVIEW_MATRIX,CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelViewIT, 
				CG_GL_MODELVIEW_MATRIX,CG_GL_MATRIX_INVERSE_TRANSPOSE);
}

// used to draw the textured envirnoment cube
void drawSkyBox( void ) {

    glBindTexture( GL_TEXTURE_CUBE_MAP_ARB, cubemapTextureID );
    glEnable( GL_TEXTURE_CUBE_MAP_ARB );

	// Initialize object linear texgen
	GLfloat s_plane[] = { 1.0, 0.0, 0.0, 0.0 };
	GLfloat t_plane[] = { 0.0, 1.0, 0.0, 0.0 };
	GLfloat r_plane[] = { 0.0, 0.0, 1.0, 0.0 };
	glTexGenfv(GL_S, GL_OBJECT_PLANE, s_plane);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, t_plane);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, r_plane);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	
	glMatrixMode(GL_MODELVIEW);
		
	glPushMatrix();
		//glLoadIdentity();
		glutSolidCube(25.0);
	glPopMatrix();

	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_TEXTURE_GEN_R);
	
	glDisable( GL_TEXTURE_CUBE_MAP_ARB );  
}


